#ifndef __UFCSVRCOM_H
#define __UFCSVRCOM_H


#include "ufsacc.h"

#if defined VMS  || defined __VMS
#pragma nomember_alignment	    /* like on VAXes */
#endif

#define UFS_VERSION  1  /* version of exchange protocol */ 
#define UFS_MANAGER_PORT 4326


int SizeOfDataTypes[12] = {
    UFS_DTSize_BitSet8,
    UFS_DTSize_BitSet1,
    UFS_DTSize_BitSet32,
    UFS_DTSize_Integer16,
    UFS_DTSize_Integer32,
    UFS_DTSize_Fix16,
    UFS_DTSize_Fix32,
    UFS_DTSize_RealF,
    UFS_DTSize_Real_t,
    UFS_DTSize_Integer8,
    UFS_DTSize_RealD,
    UFS_DTSize_Structure,
    };

#define INET_C_DSC_ALL 2



#define MBXPortNrName	"PortNr_MBX"
#define MailBoxName	"UFSLOG_MBX"

#define DATASIZE     16000


/* type of packets */
#define UFS_PKT_none          0	/* do not use number 0 */
#define UFS_PKT_svrCrea       1 /* create a new server */
#define UFS_PKT_endSvr        2
#define UFS_PKT_svrStat       3
#define UFS_PKT_clntId        4 
#define UFS_PKT_devAcc        5
#define UFS_PKT_devRsp        6

#define UFS_PKT_numExch      99     /* for test purposes only */


/* types of device access */
#define UFS_ACC_none          0	/* do not use number 0 */
#define UFS_ACC_Read          1
#define UFS_ACC_Write         2
#define UFS_ACC_Call          3


/* server may indicate these messages */
#define UFS_MSG_none          0
#define UFS_MSG_ok            1
#define UFS_MSG_unknownType   2
#define UFS_MSG_packetToBig   3


/* definitions of data to be exchanged */

/* request for device access */
/* device access header */
typedef struct {
  /* all non 4-byte type must be first (for little-big endian swap) */
  UFS_nomen	nomen;
  UFS_property	property;
  UFS_pClass	pClass;
} UFS_T_devAccHdrBy;

typedef struct {
  UFS_INTS4     vrtAcc;
  UFS_INTS4     dCnt;
  UFS_INTS4     dTyp;
  UFS_INTS4     pCnt;
  UFS_INTS4     pTyp;
} UFS_T_devAccHdrLg;

typedef struct {
  /* all non 4-byte type must be first (for little-big endian swap) */
  UFS_T_devAccHdrBy by;
  UFS_T_devAccHdrLg lg;
} UFS_T_devAccHdr;


/* device access packet */
typedef struct {
  UFS_T_devAccHdr hdr;
  UFS_CHARS parDat [DATASIZE];
} UFS_T_devAcc;


/* response to device access */
/* device response header */
typedef struct {
  UFS_INTU4   execStat;  /* summary status of dev access */
  UFS_INTU4   ufcStat;   /* userface status */
  UFS_usrCmpl usrCmpl;   /* usr completion information */
  UFS_INTS4   dRdCnt;
} UFS_T_devRspHdr;

/* response to device access packet */
typedef struct {
  UFS_T_devRspHdr hdr;
  UFS_CHARS parDat [DATASIZE];
} UFS_T_devRsp;


/* information from server */
typedef struct ServerStat {
  UFS_INTS4 status;
} UFS_T_svrStat;



/* !temporary! */
typedef struct numExchange {
  UFS_INTS4 cnt;
  UFS_INTS4 data[8];
} UFS_T_NumExch;

#define USER_NAME_LENGTH 32
#define PASSWD_NAME_LENGTH 32
#define HOST_NAME_LENGTH 64

/* user information */
typedef struct {
  UFS_CHARS user[USER_NAME_LENGTH];
  UFS_CHARS passwd[PASSWD_NAME_LENGTH]; /* not yet used */
  UFS_CHARS host[HOST_NAME_LENGTH];
  UFS_INTS4 ipAddress;
} UFS_T_clntId;


/* data of a new server */
typedef struct {
    UFS_INTS4 createStatus;
    UFS_INTS4 portNumber;
} UFS_T_svrCrea;

/*data to stop server*/
typedef struct {
    UFS_INTS4 status;
} UFS_T_endSvr;

/* every packet MUST start like this */
typedef struct Header {
  UFS_INTS4 endian;   /* low/high endian same as sender?        (4 byte) */
  UFS_INTS4 version;  /* version of communication               (4 byte) */
  UFS_INTS4 pckType;  /* type of data to be exchanged           (4 byte) */
  UFS_INTS4 plSize;   /* size of following data to be exchanged (4 byte) */
} UFS_T_Hdr;


/* data to be exchanged, variable structure */
typedef union Payload {
  UFS_T_clntId    clntId;
  UFS_T_svrCrea   svrCrea;
  UFS_T_endSvr    endSvr;
  UFS_T_svrStat   svrStat;
  UFS_T_devAcc    devAcc;
  UFS_T_devRsp    devRsp;
} UFS_T_Pl;


/* packets exchanged client <-> server */
typedef struct UfcSvrPacket {
  UFS_T_Hdr  hdr;
  UFS_T_Pl   pl;
} UFS_T_Pkt;


UFS_T_Pkt  *UFS_T_PktPtr;



#endif	/* __UFCSVRCOM_H */
