#ifndef _UFSACC_H
#define _UFSACC_H

#ifdef __cplusplus
extern "C" {
#endif

#ifdef WIN32
/* The following ifdef block is the standard way of creating macros which    */
/* make exporting from a DLL simpler. All files within this DLL are compiled */
/* with the UFS_DLL_EXPORTS symbol defined on the command line. this symbol  */
/* should not be defined on any project that uses this DLL. This way any     */
/* other project whose source files include this file see UFS_DLL_API        */
/* functions as being imported from a DLL, wheras this DLL sees symbols      */
/* defined with this macro as being exported.                                */
#ifdef UFSACC_EXPORTS
#define UFSACC_API __declspec(dllexport)
#else
#define UFSACC_API __declspec(dllimport)
#endif

#else
#define UFSACC_API /**/
#endif





#if defined(VMS) || defined(__VMS)
#include <netdb.h>
#include <socket.h>         /* tcp/ip */
#include <ucx$inetdef.h>    /* UCX network definitions */
#include <unixio.h>
#else
#ifdef WIN32
#include <winsock.h>
#else
#include  <netdb.h>
#ifdef Lynx
#include <socket.h>
#include <unistd.h>
#else
#include  <sys/socket.h>
#include  <sys/unistd.h>
#endif
#include  <netinet/in.h>
#endif
#endif



typedef          char   UFS_CHARS;
typedef unsigned char   UFS_CHARU;
typedef          int    UFS_INTS4;
typedef unsigned int    UFS_INTU4;
typedef          float  UFS_REAL4;
typedef          double UFS_REAL8;



/* supported UFC data types */
#define UFS_DT_BitSet8	    0
#define UFS_DT_BitSet16	    1
#define UFS_DT_BitSet32	    2
#define UFS_DT_Integer16    3
#define UFS_DT_Integer32    4
#define UFS_DT_Fix16	    5
#define UFS_DT_Fix32	    6
#define UFS_DT_RealF	    7
#define UFS_DT_Real_t	    8
#define UFS_DT_Integer8	    9
#define UFS_DT_RealD	   10
#define UFS_DT_Structure   11

#define UFS_DTSize_BitSet8	1
#define UFS_DTSize_BitSet1	2 
#define UFS_DTSize_BitSet32	4
#define UFS_DTSize_Integer16    2
#define UFS_DTSize_Integer32    4
#define UFS_DTSize_Fix16	2
#define UFS_DTSize_Fix32	4
#define UFS_DTSize_RealF	4
#define UFS_DTSize_Real_t	4
#define UFS_DTSize_Integer8	1
#define UFS_DTSize_RealD	8
#define UFS_DTSize_Structure    1

/* type of errors */

/* Allgemeine */
#define UFS_EXIT_NORMAL		    0

/* Connect */
#define UFS_EXIT_CORRUPT           -1
#define UFS_EXIT_VERSION_DIFFER    -2
#define UFS_EXIT_WSA_STARTUP       -3
#define UFS_EXIT_SOCKET_STARTER    -4
#define UFS_EXIT_CONN_STARTER      -5
#define UFS_EXIT_SEND_STARTER      -6
#define UFS_EXIT_SERVER_START      -7
#define UFS_EXIT_SOCKET            -8
#define UFS_EXIT_CONNECT           -9
#define UFS_EXIT_NO_ACCESS_SERVER -10
#define UFS_EXIT_RESPONSE         -11
#define UFS_EXIT_SEND             -12
#define UFS_EXIT_STOP_SERVER      -13

/* access */
#define UFS_EXIT_UFC_ERROR          1  /* other userface error */
#define UFS_EXIT_USR_ERROR          2  /* error in usr execution */

#define UFS_EXIT_NO_ACCESS_DEVICE  10  /* no access to device */
#define UFS_EXIT_OFFLINE           11  /* device is offline */
#define UFS_EXIT_TIMEOUT           12  /* timeout during device access */
#define UFS_EXIT_UNKNOWN_NOMEN     13  /* unknown nomenclature */
#define UFS_EXIT_UNKNOWN_PROP      14  /* unknown property/propclass */
#define UFS_EXIT_VIRT_ACCEL        15  /* illegal virtual accelerator */
#define UFS_EXIT_DATACOUNT         16  /* more receive data than requested */
#define UFS_EXIT_DATA_CONVER       17  /* cannot convert data to/from device format */
#define UFS_EXIT_PARA_CONVER       18  /* cannot convert para to device format */





#define UFS_NOMEN_LENGTH  12
#define UFS_PROP_LENGTH   12
#define UFS_PCLASS_LENGTH 8

typedef UFS_CHARS UFS_nomen    [UFS_NOMEN_LENGTH];
typedef UFS_CHARS UFS_property [UFS_PROP_LENGTH];
typedef UFS_CHARS UFS_pClass   [UFS_PCLASS_LENGTH];

//typedef struct {
//  UFS_INTS4	typ;          /* datatype (UFS_DT_...) */
//  UFS_INTS4	cnt;         /* data count */
//  UFS_CHARS	*dat;	    /* data */
//  } UFS_data;
//typedef UFS_data *UFS_dataPtr;


typedef struct {
    UFS_INTU4   usrPrimStat;
    UFS_INTU4   usrSecStat;   
    UFS_INTU4   eventStamp;
    UFS_INTU4   timeStamp[4];
    UFS_INTS4   energy;
    UFS_INTS4   focus;
    UFS_INTS4   intensity;
    UFS_INTS4   cycleCnt;
    UFS_INTU4   dataId;
    UFS_INTS4   USRSD_res1;
    UFS_INTS4   USRSD_res2;
    UFS_INTS4   USRSD_res3;
  } UFS_usrCmpl;

typedef UFS_usrCmpl *UFS_usrCmplPtr;




/* Connection to server, return sockedId or error codes */
UFSACC_API UFS_INTS4 UFS_connect(UFS_CHARS host_address[]);

/* Disconnect from sever */
UFSACC_API UFS_INTS4 UFS_close(UFS_INTS4 socketUsed);

UFSACC_API UFS_INTS4 UFS_access(UFS_INTS4 socketUsed,
             UFS_nomen nom, UFS_property prop,
     	     UFS_pClass pClass, UFS_INTS4 vrtAcc, 
             UFS_INTS4 para_cnt, UFS_INTS4 para_typ, void *para,
             UFS_INTS4 data_cnt, UFS_INTS4 data_typ, void *data,
             UFS_INTS4 *data_rd_cnt,
             UFS_INTU4 *ufcStat, UFS_usrCmpl *usrCmpl);

UFSACC_API UFS_INTS4 UFS_ret_txt(UFS_INTS4 ret_code, UFS_CHARS text[]);


#ifdef __cplusplus
}
#endif  /* __cplusplus */

#endif
